#include  <bsp_os.h>
#include  "bsp.h"

#include  <cpu/include/cpu.h>
#include  <common/include/common.h>
#include  <kernel/include/os.h>
#include  <kernel/include/os_trace.h>

#include  <common/include/lib_def.h>
#include  <common/include/rtos_utils.h>
#include  <common/include/toolchains.h>
#include  "retargetserial.h"

/********************************************************************************************************
*                                             LOCAL DEFINES
********************************************************************************************************/
#define  START_TASK_PRIO              21u								// Start Task priority
#define  START_TASK_STK_SIZE         256u								// Start Task stack size
#define  MY_TASK_STK_SIZE			 256u

/********************************************************************************************************
*                                        LOCAL GLOBAL VARIABLES
********************************************************************************************************/
static  CPU_STK StartTaskStk[START_TASK_STK_SIZE];  					// Start Task Stack.
static  CPU_STK	FirstTaskStk[MY_TASK_STK_SIZE];
static  CPU_STK	SecondTaskStk[MY_TASK_STK_SIZE];

static  OS_TCB  StartTaskTCB;											// Start Task TCB.
static  OS_TCB	FirstTaskTCB;
static  OS_TCB	SecondTaskTCB;

static 	CPU_INT32U      MyVar;
static 	OS_MUTEX      	MyMutex;

/*********************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************/
static void StartTask(void *p_arg);										// Start Task prototype
static void	FirstTask (void *p);
static void	SecondTask (void *p);

int main (void)
{
    RTOS_ERR  err;

    BSP_SystemInit();                                           		// Initialize System.
    CPU_Init();                                                 		// Initialize CPU.

    RETARGET_SerialInit();
    RETARGET_SerialCrLf(1);

    OS_TRACE_INIT();
    OSInit(&err);                                               		// Initialize the Kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSTaskCreate(&StartTaskTCB,                          				// Create the Start Task.
                 "Start Task",
                  StartTask,
                  DEF_NULL,
                  START_TASK_PRIO,
                 &StartTaskStk[0],
                 (START_TASK_STK_SIZE / 10u),
                  START_TASK_STK_SIZE,
                  0u,
                  0u,
                  DEF_NULL,
                 (OS_OPT_TASK_STK_CLR),
                 &err);
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSStart(&err);                                              		// Start the kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    return (1);
}

static void StartTask (void *p_arg)
{
    RTOS_ERR  err;
    PP_UNUSED_PARAM(p_arg);                                     		// Prevent compiler warning.
    BSP_TickInit();                                             		// Initialize Kernel tick source.

#if (OS_CFG_STAT_TASK_EN == DEF_ENABLED)
    OSStatTaskCPUUsageInit(&err);                               		// Initialize CPU Usage.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;);	//   Check error code.
#endif

#ifdef CPU_CFG_INT_DIS_MEAS_EN
    CPU_IntDisMeasMaxCurReset();                                		// Initialize interrupts disabled measurement.
#endif

    Common_Init(&err);                                          		// Call common module initialization example.
    APP_RTOS_ASSERT_CRITICAL(err.Code == RTOS_ERR_NONE, ;);

    BSP_OS_Init();                                              		// Initialize the BSP. It is expected that the BSP ...
                                                                		// ... will register all the hardware controller to ...
                                                                		// ... the platform manager at this moment.
    GPIO_PinModeSet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN, gpioModeInput, 0);
//    GPIO_PinModeSet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB1_PIN, gpioModeInput, 0);

    OSMutexCreate (&MyMutex, "My mutex", &err);					// init semaphore object

    OSTaskCreate(&FirstTaskTCB,           						// Create first custom tasks
                 "First Task",
                 FirstTask,
                 0u,
				 START_TASK_PRIO+2,
                 &FirstTaskStk[0u],
                 FirstTaskStk[MY_TASK_STK_SIZE / 10u],
                 MY_TASK_STK_SIZE,
                 0u,
                 0u,
                 0u,
                 (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                 &err);

    OSTaskCreate(&SecondTaskTCB,              					// Create second custom task
                 "Second Task",
                  SecondTask,
                  0u,
                  START_TASK_PRIO+1,
                  &SecondTaskStk[0u],
                  SecondTaskStk[MY_TASK_STK_SIZE / 10u],
                  MY_TASK_STK_SIZE,
                  0u,
                  0u,
                  0u,
                  (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                  &err);

    while (1)
    {
        OSTimeDlyHMSM(1u, 0u, 0u, 0u, OS_OPT_TIME_HMSM_STRICT, &err);
        APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;); //   Check error code.
    }
}

static void FirstTask (void *p_arg)
{
    RTOS_ERR err;
    PP_UNUSED_PARAM(p_arg);                                     // Prevent compiler warning
    uint32_t button_pressed, button_state = 1;					// 1 = off, 0 = pressed

    while (DEF_TRUE)
    {
    	button_pressed = GPIO_PinInGet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN); // current button state, 0 = pressed
    	if (button_state == 1)          						// old state = "OFF"
    	{
    		if (button_pressed == 0)
    		{													// button press is detected
    			button_state = 0;             					// new status
    	    	OSMutexPend(&MyMutex, 0, OS_OPT_PEND_BLOCKING, 0, &err);
    	    	MyVar = (MyVar + 1) % 101;						// increment MyVar
    	    	OSMutexPost(&MyMutex, OS_OPT_POST_NONE, &err);
    	    }
    	}
    	else                           							// old status is "ON"
    	{
    		if (button_pressed != 0)
    		{                            						// button release is detected
    	       button_state = 1;           						// just update button state
    	    }
    	}

    	OSTimeDlyHMSM(0u, 0u, 0u, 50u,
    	                      OS_OPT_TIME_HMSM_STRICT, &err);	// 50 msec delay

    }
}

static void SecondTask (void *p_arg)
{
    RTOS_ERR err;
    PP_UNUSED_PARAM(p_arg);                                     // Prevent compiler warning

    while (DEF_TRUE)
    {
    	OSMutexPend(&MyMutex, 0, OS_OPT_PEND_BLOCKING, 0, &err);	// wait for semaphore
    	MyVar = (MyVar + 5) % 101;
    	OSMutexPost(&MyMutex, OS_OPT_POST_NONE, &err);
    	OSTimeDlyHMSM(0u, 0u, 1u, 0u,
    	                      OS_OPT_TIME_HMSM_STRICT, &err);	// 1 second delay
    }
}
